Components.utils.import("resource://gre/modules/NetUtil.jsm");
Components.utils.import("resource://gre/modules/FileUtils.jsm");

var gENForward = {
	en_email: null,
	msgCompFields: null,
	msgSend: null,
	mimeEncoder: null,
	uConv: null,
	dirService: null,
	selectedMsg: null,
	id: null,
	totalMsgs: 0,
	sentMsgs: 0,
	
	init: function() {
		this.msgSend = Components.classes["@mozilla.org/messengercompose/send;1"]
			.createInstance(Components.interfaces.nsIMsgSend);
		this.mimeEncoder = Components.classes["@mozilla.org/messenger/mimeconverter;1"]
			.getService(Components.interfaces.nsIMimeConverter);
		this.uConv = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"]
			.getService(Components.interfaces.nsIScriptableUnicodeConverter);
		this.dirService =  Components.classes["@mozilla.org/file/directory_service;1"]
			.getService(Components.interfaces.nsIProperties);
		this.tagService = Components.classes["@mozilla.org/messenger/tagservice;1"]
	  .getService(Components.interfaces.nsIMsgTagService);
	  this.accountManager = Components.classes["@mozilla.org/messenger/account-manager;1"].
			getService(Components.interfaces.nsIMsgAccountManager);
	},
	
	forwardSelectedMsgs: function(first) {
		if (first) {
			this.en_email = nsPreferences.copyUnicharPref("extensions.enforward.email", "");
			if (!this.en_email) {
				document.getElementById("statusText").setAttribute("label", "Please input your Evernote email address in ENForward settings");
				return;
			}
			this.selectedMsgs = gFolderDisplay.selectedMessages;
			var server = gFolderDisplay.displayedFolder.server;
			this.id = null;
			if (server && (server.type == "imap" || server.type == "pop3")) {
				this.id = getIdentityForServer(server);
			} else { //use default identity
				this.id = this.accountManager.defaultAccount.defaultIdentity;
			}
			if (!this.id) {
				document.getElementById("statusText").setAttribute("label", "Could not find outgoing server setting.");
				return;
			}
			this.sentMsgs = 0;
			this.totalMsgs = this.selectedMsgs.length;
		}

		var msgHdr = this.selectedMsgs.shift();
		if (msgHdr) {
			var result = this.forwardMsg(msgHdr, this.id);
			if (!result) document.getElementById("statusText").setAttribute("label", "Sending note canceled.");
		}
	},
	
	forwardMsg: function(msgHdr, id) {
		if (!this.checkMsgSize(msgHdr)) { //file size error
			this.sentMsgs += 1;
			document.getElementById("statusText").setAttribute("label", "Note size exceeds the limit. Canceled.");
		} else if (this.checkLimitExpires() || this.checkSentTimes()) {
			this.msgCompFields = Components.classes["@mozilla.org/messengercompose/composefields;1"]
				.createInstance(Components.interfaces.nsIMsgCompFields);
			this.msgCompFields.from = id.email;
			this.msgCompFields.to = this.en_email;
			if (!nsPreferences.getBoolPref("extensions.enforward.save_sent", true)) {
				this.msgCompFields.fcc = "nocopy://";
				this.msgCompFields.fcc2 = "nocopy://";
			}
			
			var prefix = nsPreferences.copyUnicharPref("extensions.enforward.prefix", "");
			var suffix = nsPreferences.copyUnicharPref("extensions.enforward.suffix", "");
			var notebook = nsPreferences.copyUnicharPref("extensions.enforward.notebook", "");
			var defaultTags = nsPreferences.copyUnicharPref("extensions.enforward.tags", "");
			var tags = nsPreferences.getBoolPref("extensions.enforward.add_msg_tags", false) ? this.getTagsForMsg(msgHdr) : [];
			if (defaultTags) {
				tags.concat(defaultTags.split(/\s*,\s*/));
			}
			this.sendMsgFile(msgHdr, prefix, suffix, notebook, tags, id);
			
			return true;
		} else { //sent times error
			document.getElementById("statusText").setAttribute("label", "You cannot send note anymore today. Canceled.");
			return false;
		}
	},
	
	getTagsForMsg: function(msgHdr) {
		var curKeys = msgHdr.getStringProperty("keywords");
		if (msgHdr.label)
		  curKeys += " $label" + msgHdr.label;
		var keys = curKeys ? curKeys.split(" ") : [];
		var tags = [];
		var len = keys.length;
		for (var i=0; i<len; i++) {
			tags.push(this.tagService.getTagForKey(keys[i]));
		}
		
		return tags;
	},
	
	encode: function(str, offset, len, charset) {
		if (!charset) charset = "UTF-8";
		var estr = this.mimeEncoder.encodeMimePartIIStr_UTF8(str, false, charset, offset, len);
		return estr;
	},
	
	checkMsgSize: function(msgHdr) {
		if (nsPreferences.getBoolPref("extensions.enforward.alert_limit", false)) {
			var size = msgHdr.messageSize;
			var maxSize = nsPreferences.getIntPref("extensions.enforward.max_size", 25);
			return size < maxSize * 1024 * 1024;
		} else {
			return true;
		}
	},
	
	checkSentTimes: function() {
		if (nsPreferences.getBoolPref("extensions.enforward.alert_limit", false)) {
			var sent = nsPreferences.getIntPref("extensions.enforward.sent_times", 0);
			var maxSend = nsPreferences.getIntPref("extensions.enforward.max_send", 50);
			
			return sent < maxSend;
		} else {
			return true;
		}
	},
	
	checkLimitExpires: function(force) {
		if (force || nsPreferences.getBoolPref("extensions.enforward.alert_limit", false)) {
			var date = new Date();
			var today = date.getFullYear() + "-" + (date.getMonth() + 1) + "-" + date.getDate();
			var last = nsPreferences.copyUnicharPref("extensions.enforward.sent_date", "");
			return today != last;
		} else {
			return true;
		}
	},
	
	saveSentDone: function() {
		var sent = 0;
		if (this.checkLimitExpires(true)) {
			sent = 1;
			var date = new Date();
			var today = date.getFullYear() + "-" + (date.getMonth() + 1) + "-" + date.getDate();
			nsPreferences.setUnicharPref("extensions.enforward.sent_date", today);
		} else {
			sent = nsPreferences.getIntPref("extensions.enforward.sent_times", 0) + 1;
		}
		nsPreferences.setIntPref("extensions.enforward.sent_times", sent);
	},
	
	getTagsString: function(tags) {
		var len = tags.length;
		var str = "";
		for (var i=0; i<len; i++) {
			if (tags[i]) str = str + " #" + tags[i];
		}
		
		return str;
	},
	
	sendMsgFile: function(msgHdr, prefix, suffix, notebook, tags, id) {
		var uri = msgHdr.folder.getUriForMsg(msgHdr);
		var msgFile = this.createTempFile();

		var messageService = messenger.messageServiceFromURI(uri);
		var messageStream = Components.classes["@mozilla.org/network/sync-stream-listener;1"].
		  createInstance().QueryInterface(Components.interfaces.nsIInputStream);
		var inputStream = Components.classes["@mozilla.org/scriptableinputstream;1"].
		  createInstance().QueryInterface(Components.interfaces.nsIScriptableInputStream);
		inputStream.init(messageStream);
		messageService.streamMessage(uri, messageStream, msgWindow, null, false, null);
		var os = Components.classes['@mozilla.org/network/file-output-stream;1'].
									createInstance(Components.interfaces.nsIFileOutputStream);
		os.init(msgFile, 2, 0x200, false); // open as "write only"

		var data = "";
		var eohInfo = null;
		while (inputStream.available()) {
			data += inputStream.read(512);
			if (!eohInfo) {
				eohInfo = this.findEndOfHeader(data);
				if (!eohInfo) continue;
				var hdr = data.substring(0, eohInfo.index).split(eohInfo.retcode);
				data = data.substring(eohInfo.index, data.length);
				var line = "";
				var ignored = false;
				while (line = hdr.shift()) {
					if ((ignored && /^\s+/.test(line))) continue;
					if (ignored = this.isIgnoredHdr(line)) continue;
					if (/^from:/i.test(line)) {
						line = "From: "+this.msgCompFields.from;
						ignored = true;
					} else if (/^to:/i.test(line)) {
						line = "To: "+this.msgCompFields.to;
						ignored = true;
					} else if (/^subject:/i.test(line)) {
						var noteStr = notebook ? "@"+notebook : "";
						var tagsStr = tags && tags.length > 0 ? this.getTagsString(tags) : "";
						var subject = prefix + msgHdr.mime2DecodedSubject + suffix + " " + noteStr + tagsStr;
						line = "Subject: "+this.encode(subject, 9, 72, msgHdr.Charset);
						ignored = true;
					}
					//write line
					line = hdr.length > 0 ? line + eohInfo.retcode : line;
					os.write(line, line.length);
				}
			} else { //now in the message body
				//write data
				os.write(data, data.length);
				data = "";
			}
		}
		messageStream.close();
		inputStream.close();
		os.close();

		var that = this;
		var sendListener = {
			onStartSending: function(aMsgID, aMsgSize) {
				that.sentMsgs += 1;
				document.getElementById("statusText").setAttribute("label", "Sending note to Evernote ... "+"["+that.sentMsgs+"/"+that.totalMsgs+"]");
			},
			
			onProgress: function(aMsgID, aProgress, aProgressMax) {
				
			},
			
			onStatus: function(aMsgID, aMsg) {
				
			},
			
			onStopSending: function(aMsgID, aStatus, aMsg, returnFileSpec) {
				if (aStatus) { //error
					document.getElementById("statusText").setAttribute("label", "Failed to send note.");
					msgFile.remove(false);
				} else {
					document.getElementById("statusText").setAttribute("label", "Done.");
					that.saveSentDone();
					msgFile.remove(false);
				}
				setTimeout(function() {
					that.forwardSelectedMsgs(false);
				}, 1000);
				//do next
			}
		};

		var msgSend = Components.classes["@mozilla.org/messengercompose/send;1"]
										.createInstance(Components.interfaces.nsIMsgSend);
		 msgSend.sendMessageFile(
			 id,                // in nsIMsgIdentity       aUserIdentity,
			 id.key,          // char* accountKey,
			 this.msgCompFields,                   // in nsIMsgCompFields     fields,
			 msgFile,                        // in nsIFile          sendIFile,
			 true,                            // in PRBool               deleteSendFileOnCompletion,
			 false,                           // in PRBool               digest_p,
			 msgSend.nsMsgDeliverNow,         // in nsMsgDeliverMode     mode,
			 null,                            // in nsIMsgDBHdr          msgToReplace,
			 sendListener,     // in nsIMsgSendListener   aListener,
			 null,   // in nsIMsgStatusFeedback aStatusFeedback,
			 null                             // in string               password
			);

	},
	
	isIgnoredHdr: function(line) {
		var ignored =	/^>*from \S+ /i.test(line) ||
			/^bcc: /i.test(line) ||
			/^fcc: /i.test(line) ||
			/^content-length: /i.test(line) ||
			/^lines: /i.test(line) ||
			/^status: /i.test(line) ||
			/^x-.+: /i.test(line) ||
			/^return-path: /i.test(line) ||
			/^delivered-to: /i.test(line) ||
			/^authentication-results: /i.test(line) ||
			/^message-id: /i.test(line) ||
			/^(?:in-)*reply-to: /i.test(line) ||
			/^bounce-to: /i.test(line) ||
			/^received(?:-.+)*: /i.test(line);
		
		return ignored;
	},
	
	findEndOfHeader: function(data) {
		var candidates = ["\r\n", "\n\r", "\n", "\r"];
		var headerEnd = -1;
		var ret = null;
		for (var i=0; i<candidates.length; i++) {
			var candidate = candidates[i];
			headerEnd = data.indexOf(candidate + candidate);
			if (headerEnd != -1) {
				ret = {index: headerEnd, retcode: candidate};
				break;
			}
		}
		
		return ret;
	},
	
	createTempFile: function() {
		var tmpDir = this.dirService.get("TmpD", Components.interfaces.nsIFile)
		tmpDir.append("enforward.tmp");
		tmpDir.createUnique(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 0666);
		return tmpDir;
	}

};

window.addEventListener("load", function(){gENForward.init()}, false);
